/*****************************************************************************\
* (c) Copyright 2020 CERN for the benefit of the LHCb Collaboration           *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
Ext.define("LHCbDIRAC.AnalysisProductions.classes.ProductionView.ProdModel", {
  extend: "Ext.data.Model",
  fields: [
    "id",
    "name",
    "outputPattern",
    "state",
    "LFNs",
    "PFNs",
    "outputSize",
    "numOutputFiles",
    "numOutputFilesAtCERN",
    "inputBKPath",
    "transformations",
    "archived",
    "published",
  ],
});
