/*****************************************************************************\
* (c) Copyright 2021 CERN for the benefit of the LHCb Collaboration           *
*                                                                             *
* This software is distributed under the terms of the GNU General Public      *
* Licence version 3 (GPL Version 3), copied verbatim in the file "COPYING".   *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
.bk-icon-go {
  background-image: url(../images/go.gif) !important;
  background-color: transparent;
}
.bk-icon-next {
  background-image: url(../images/next.png) !important;
  background-color: transparent;
}
.bk-icon-back {
  background-image: url(../images/back.png) !important;
  background-color: transparent;
}
.bk-icon-close {
  background-image: url(../images/close.png) !important;
  background-color: transparent;
}
